package com.flyfish.framework.domain.authorized;

import com.flyfish.framework.annotations.Property;
import com.flyfish.framework.context.UserContext;
import com.flyfish.framework.domain.base.AuditDomain;
import com.flyfish.framework.domain.po.Department;
import com.flyfish.framework.domain.po.User;
import com.flyfish.framework.exception.biz.InvalidBusinessException;
import lombok.Setter;
import org.apache.commons.lang3.StringUtils;

import javax.validation.constraints.NotBlank;

/**
 * 带鉴权的实体，主要以部门隔绝
 */
@Setter
public abstract class AuthorizedDomain extends AuditDomain {

    // 作用域id，一般是部门。用户存储时插入
    @NotBlank(message = "请选择当前部")
    @Property(readonly = true)
    private String authorizeId;

    public String getAuthorizeId() {
        if (StringUtils.isNotBlank(authorizeId)) {
            return authorizeId;
        } else {
            return UserContext.sharedContext()
                    .map(UserContext::currentUser)
                    .map(User::getAuthority)
                    .filter(StringUtils::isNotBlank)
                    .orElse(Department.PUBLIC);
        }
    }
}
