package com.flyfish.framework.domain.authorized;

import com.flyfish.framework.builder.CriteriaBuilder;
import com.flyfish.framework.context.SpringContext;
import com.flyfish.framework.domain.base.NameLikeQo;
import com.flyfish.framework.domain.po.Department;
import com.flyfish.framework.enums.UserType;
import com.flyfish.framework.service.DepartmentService;
import lombok.Getter;
import lombok.Setter;
import lombok.val;
import org.apache.commons.collections4.CollectionUtils;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 带鉴权的查询实体，主要以部门隔绝
 *
 * @param <T>
 */
@Getter
@Setter
public abstract class AuthorizedQo<T extends AuthorizedDomain> extends NameLikeQo<T> {

    // 部门服务
    private DepartmentService departmentService;

    /**
     * 获取可见的权限ids
     *
     * @return 结果
     */
    public List<String> getAuthorizedIds() {
        if (null != user) {
            // 是超级管理员，放行，返回null
            if (user.getType() == UserType.SUPER_ADMIN) {
                return null;
            }
            if (CollectionUtils.isNotEmpty(user.getDepartments())) {
                val departs = user.getDepartments().stream().map(Department::getId)
                        .collect(Collectors.toList());
                val result = getSubAuthorities(departs);
                if (CollectionUtils.isNotEmpty(result)) {
                    return result;
                }
            }
        }
        return Collections.singletonList(Department.PUBLIC);
    }

    @Override
    public CriteriaBuilder<T> criteriaBuilder() {
        return super.criteriaBuilder().with("authorizedIds", "authorizeId", CriteriaBuilder.Builders.IN);
    }

    private List<String> getSubAuthorities(List<String> parents) {
        if (null == departmentService) {
            departmentService = SpringContext.getBean(DepartmentService.class);
        }
        return departmentService.getSubCodes(parents);
    }
}
